@@ -1,10 +1,10 @@
 # vim: set fileencoding=utf-8 :
 
 import ioc.exceptions, ioc.helper
-import re, exceptions
-import importlib, inspect
 from ioc.proxy import Proxy
 
+import importlib, inspect, re
+
 class Extension(object):
     def load(self, config, container_builder):
         pass
@@ -185,8 +185,8 @@ class ContainerBuilder(Container):
             self.logger.debug("Start building the container")
 
         extensions = []
-
         container.add("service_container", container)
+        self.parameters.set('ioc.extensions', self.extensions.keys())
 
         for name, config in self.extensions.iteritems():
             name = "%s.di.Extension" % name
@@ -199,6 +199,7 @@ class ContainerBuilder(Container):
 
             extensions.append(extension)
 
+
         for extension in extensions:
             extension.post_load(self, container)
 
@@ -223,7 +224,7 @@ class ContainerBuilder(Container):
             )
 
         if self.logger:
-            self.logger.debug("End resolving all parameters!")
+            self.logger.debug("End resolving all parameters! Yeah!!!!")
 
         if container.has('ioc.extra.event_dispatcher'):
             container.get('ioc.extra.event_dispatcher').dispatch('ioc.container.built', {
@@ -252,7 +253,9 @@ class ContainerBuilder(Container):
 
         return clazz
 
-    def get_instance(self, klass, definition, container):
+    def get_instance(self, definition, container):
+
+        klass = self.get_class(definition)
 
         if self.logger:
             self.logger.debug("Create instance for %s" % klass)
@@ -261,7 +264,7 @@ class ContainerBuilder(Container):
             args = self.set_services(definition.arguments, container)
             kwargs = self.set_services(definition.kwargs, container)
 
-            instance = klass(*args, **kwargs)
+            instance = klass(*args, **kwargs)            
         else:
             # module object ...
             instance = klass
@@ -300,7 +303,7 @@ class ContainerBuilder(Container):
             raise ioc.exceptions.CyclicReference(" -> ".join(self.stack) + " -> " + id)
 
         self.stack.append(id)
-        instance = self.get_instance(self.get_class(definition), definition, container)
+        instance = self.get_instance(definition, container)
         container.add(id, instance)
         self.stack.pop()
 
@@ -322,6 +325,9 @@ class ContainerBuilder(Container):
         if isinstance(value, Reference) and container.has(value.id):
             return container.get(value.id)
 
+        if isinstance(value, Definition):
+            return self.get_instance(value, container)
+
         if ioc.helper.is_iterable(value):
             return self.set_services(value, container)
 
